@@ -1,10 +1,17 @@
-from ..base import *
+from easymlops.table.core import *
+from easymlops.nlp.core import *
 import numpy as np
 import pandas as pd
+from easymlops.table.utils import PandasUtils
 
 
-class RepresentationBase(PipeObject):
+class RepresentationBase(TablePipeObjectBase):
+    """
+    所有文本特征提取类的基础类
+    """
+
     def __init__(self, cols="all", skip_check_transform_type=True, native_init_params=None, native_fit_params=None,
+                 drop_input_data=True,
                  **kwargs):
         super().__init__(skip_check_transform_type=skip_check_transform_type, **kwargs)
         self.cols = cols
@@ -15,52 +22,36 @@ class RepresentationBase(PipeObject):
             self.native_init_params = dict()
         if self.native_fit_params is None:
             self.native_fit_params = dict()
+        self.drop_input_data = drop_input_data
 
-    def before_fit(self, s: dataframe_type) -> dataframe_type:
-        s = super().before_fit(s)
+    def before_fit(self, s: dataframe_type, **kwargs) -> dataframe_type:
+        s = super().before_fit(s, **kwargs)
         if str(self.cols).lower() in ["none", "all", "null", "nan"]:
             self.cols = self.input_col_names
         assert type(self.cols) == list and type(self.cols[0]) == str
         return s
 
-    def after_transform(self, s: dataframe_type) -> dataframe_type:
+    def after_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
         self.output_col_names = list(s.columns)
-        s = s.fillna(0)
-        return s
+        return s.fillna(0)
 
-    def after_transform_single(self, s: dict_type) -> dict_type:
-        self.extract_dict(s, self.output_col_names)
-        for key, value in s.items():
-            if str(value).lower() in ["none", "null", "nan"]:
-                s[key] = 0
-        return s
+    def transform_single(self, s: dict_type, **kwargs):
+        return self.transform(pd.DataFrame([s])).to_dict("record")[0]
 
-    def _transform_single(self, s: dict_type):
-        return self._transform(pd.DataFrame([s])).to_dict("record")[0]
-
-    def _get_params(self):
+    def udf_get_params(self):
         return {"cols": self.cols, "native_init_params": self.native_init_params,
-                "native_fit_params": self.native_fit_params}
+                "native_fit_params": self.native_fit_params, "drop_input_data": self.drop_input_data}
 
-    def _set_params(self, params: dict_type):
+    def udf_set_params(self, params: dict_type):
         self.cols = params["cols"]
         self.native_init_params = params["native_init_params"]
         self.native_fit_params = params["native_fit_params"]
+        self.drop_input_data = params["drop_input_data"]
 
 
 class BagOfWords(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    ---------------------------
-    output type:pandas.dataframe
-    output like:
-    | i |love|eat|apple|china|
-    | 1 |  1 | 1 |  1  |  0  |
-    | 1 |  1 | 0 |  0  |  1  |
+    词袋模型
     """
 
     def __init__(self, cols="all", prefix="bag", **kwargs):
@@ -79,7 +70,7 @@ class BagOfWords(RepresentationBase):
     def preprocessor_func(x):
         return x
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from sklearn.feature_extraction.text import CountVectorizer
         for col in self.cols:
             tf = CountVectorizer(**self.default_native_init_params)
@@ -87,9 +78,9 @@ class BagOfWords(RepresentationBase):
             self.tf[col] = tf
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
         # 先聚合稀疏矩阵，在添加稠密矩阵
-        dfs = []
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             tf = self.tf.get(col)
             tf_vectors_csr = tf.transform(s[col])
@@ -97,36 +88,23 @@ class BagOfWords(RepresentationBase):
                 feature_names = tf.get_feature_names()
             except:
                 feature_names = tf.get_feature_names_out()
-            df = pandas.DataFrame.sparse.from_spmatrix(data=tf_vectors_csr,
-                                                       columns=["{}_{}_{}".format(self.prefix, col, name) for name in
-                                                                feature_names])
+            df = pd.DataFrame.sparse.from_spmatrix(data=tf_vectors_csr,
+                                                   columns=["{}_{}_{}".format(self.prefix, col, name) for name in
+                                                            feature_names], index=s.index)
             dfs.append(df)
-        combine_df = pd.concat(dfs)
-        combine_cols = s.columns.tolist() + combine_df.columns.tolist()
-        for col in s.columns:
-            combine_df[col] = s[col]
-        return combine_df[combine_cols]
 
-    def _get_params(self) -> dict_type:
+        return PandasUtils.concat_duplicate_columns(dfs)
+
+    def udf_get_params(self) -> dict_type:
         return {"tf": self.tf}
 
-    def _set_params(self, params: dict_type):
+    def udf_set_params(self, params: dict_type):
         self.tf = params["tf"]
 
 
 class TFIDF(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    -----------------------------
-    output type:pandas.dataframe
-    output like:
-    | i |love|eat|apple|china|
-    |0.2|0.3 |0.4| 0.5 |  0  |
-    |0.3|0.2 | 0 |  0  | 0.2 |
+    TFIDF模型
     """
 
     def __init__(self, cols="all", prefix="tfidf", **kwargs):
@@ -145,7 +123,7 @@ class TFIDF(RepresentationBase):
     def preprocessor_func(x):
         return x
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from sklearn.feature_extraction.text import TfidfVectorizer
         for col in self.cols:
             tfidf = TfidfVectorizer(**self.default_native_init_params)
@@ -153,8 +131,8 @@ class TFIDF(RepresentationBase):
             self.tfidf[col] = tfidf
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
-        dfs = []
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             tfidf = self.tfidf.get(col)
             tfidf_vectors_csr = tfidf.transform(s[col])
@@ -162,36 +140,23 @@ class TFIDF(RepresentationBase):
                 feature_names = tfidf.get_feature_names()
             except:
                 feature_names = tfidf.get_feature_names_out()
-            df = pandas.DataFrame.sparse.from_spmatrix(data=tfidf_vectors_csr,
-                                                       columns=["{}_{}_{}".format(self.prefix, col, name) for name in
-                                                                feature_names])
+            df = pd.DataFrame.sparse.from_spmatrix(data=tfidf_vectors_csr,
+                                                   columns=["{}_{}_{}".format(self.prefix, col, name) for name in
+                                                            feature_names], index=s.index)
             dfs.append(df)
-        combine_df = pd.concat(dfs)
-        combine_cols = s.columns.tolist() + combine_df.columns.tolist()
-        for col in s.columns:
-            combine_df[col] = s[col]
-        return combine_df[combine_cols]
 
-    def _get_params(self) -> dict_type:
+        return PandasUtils.concat_duplicate_columns(dfs)
+
+    def udf_get_params(self) -> dict_type:
         return {"tfidf": self.tfidf}
 
-    def _set_params(self, params: dict_type):
+    def udf_set_params(self, params: dict_type):
         self.tfidf = params["tfidf"]
 
 
 class LdaTopicModel(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    ----------------------------
-    output type:pandas.dataframe
-    output like:
-    | 0 | 1 | 2 | 3 |
-    |0.2|0.3|0.4|0.5|
-    |0.3|0.2|0.6|0.1|
+    LDA主题模型
     """
 
     def __init__(self, cols="all", num_topics=10, prefix="lda", **kwargs):
@@ -201,7 +166,7 @@ class LdaTopicModel(RepresentationBase):
         self.common_dictionary = {}
         self.lda_model = {}
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from gensim.corpora.dictionary import Dictionary
         from gensim.models.ldamulticore import LdaModel
         for col in self.cols:
@@ -214,8 +179,9 @@ class LdaTopicModel(RepresentationBase):
             self.common_dictionary[col] = common_dictionary
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
         from gensim import matutils
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             common_dictionary = self.common_dictionary[col]
             lda_model = self.lda_model[col]
@@ -224,33 +190,23 @@ class LdaTopicModel(RepresentationBase):
             common_corpus = [common_dictionary.doc2bow(text) for text in texts]
             vectors = matutils.corpus2dense(lda_model[common_corpus],
                                             num_terms=self.native_init_params.get("num_topics")).T
-            result = pandas.DataFrame(vectors)
+            result = pd.DataFrame(vectors, index=s.index)
             result.columns = ["{}_{}_{}".format(self.prefix, col, name) for name in result.columns]
-            for icol in result.columns:
-                s[icol] = result[icol].values
-        return s
+            dfs.append(result)
 
-    def _get_params(self) -> dict_type:
+        return PandasUtils.concat_duplicate_columns(dfs)
+
+    def udf_get_params(self) -> dict_type:
         return {"common_dictionary": self.common_dictionary, "lda_model": self.lda_model}
 
-    def _set_params(self, params: dict):
+    def udf_set_params(self, params: dict):
         self.common_dictionary = params["common_dictionary"]
         self.lda_model = params["lda_model"]
 
 
 class LsiTopicModel(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    ----------------------------
-    output type:pandas.dataframe
-    output like:
-    | 0 | 1 | 2 | 3 |
-    |0.2|0.3|0.4|0.5|
-    |0.3|0.2|0.6|0.1|
+    LSI主题模型
     """
 
     def __init__(self, cols="all", num_topics=10, prefix="lsi", **kwargs):
@@ -260,7 +216,7 @@ class LsiTopicModel(RepresentationBase):
         self.common_dictionary = {}
         self.lsi_model = {}
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from gensim.corpora.dictionary import Dictionary
         from gensim.models import LsiModel
         for col in self.cols:
@@ -273,8 +229,9 @@ class LsiTopicModel(RepresentationBase):
             self.common_dictionary[col] = common_dictionary
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
         from gensim import matutils
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             lsi_model = self.lsi_model[col]
             common_dictionary = self.common_dictionary[col]
@@ -283,33 +240,22 @@ class LsiTopicModel(RepresentationBase):
             common_corpus = [common_dictionary.doc2bow(text) for text in texts]
             vectors = matutils.corpus2dense(lsi_model[common_corpus],
                                             num_terms=self.native_init_params.get("num_topics")).T
-            result = pandas.DataFrame(vectors)
+            result = pd.DataFrame(vectors, index=s.index)
             result.columns = ["{}_{}_{}".format(self.prefix, col, name) for name in result.columns]
-            for icol in result.columns:
-                s[icol] = result[icol].values
-        return s
+            dfs.append(result)
+        return PandasUtils.concat_duplicate_columns(dfs)
 
-    def _get_params(self) -> dict_type:
+    def udf_get_params(self) -> dict_type:
         return {"common_dictionary": self.common_dictionary, "lsi_model": self.lsi_model}
 
-    def _set_params(self, params: dict):
+    def udf_set_params(self, params: dict):
         self.common_dictionary = params["common_dictionary"]
         self.lsi_model = params["lsi_model"]
 
 
 class Word2VecModel(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    ----------------------------
-    output type:pandas.dataframe
-    output like:
-    | 0 | 1 | 2 | 3 |
-    |0.2|0.3|0.4|0.5|
-    |0.3|0.2|0.6|0.1|
+    Word2Vec词向量模型
     """
 
     def __init__(self, cols="all", embedding_size=16, min_count=5, prefix="w2v", **kwargs):
@@ -318,7 +264,7 @@ class Word2VecModel(RepresentationBase):
         self.native_init_params.update(self.native_fit_params)
         self.native_init_params.update({"vector_size": embedding_size, "min_count": min_count})
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from gensim.models import Word2Vec
         for col in self.cols:
             texts = s[col].values.tolist()
@@ -327,7 +273,8 @@ class Word2VecModel(RepresentationBase):
             self.w2v_model[col] = w2v_model
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             w2v_model = self.w2v_model[col]
             texts = s[col].values.tolist()
@@ -335,32 +282,22 @@ class Word2VecModel(RepresentationBase):
             vectors = [np.mean(np.asarray([w2v_model.wv[word] for word in line if word in w2v_model.wv]),
                                axis=0) + np.zeros(shape=(self.native_init_params.get("vector_size"),))
                        for line in texts]
-            result = pandas.DataFrame(vectors)
+            result = pd.DataFrame(vectors, index=s.index)
             result.columns = ["{}_{}_{}".format(self.prefix, col, name) for name in result.columns]
-            for icol in result.columns:
-                s[icol] = result[icol].values
-        return s
+            dfs.append(result)
 
-    def _get_params(self) -> dict_type:
+        return PandasUtils.concat_duplicate_columns(dfs)
+
+    def udf_get_params(self) -> dict_type:
         return {"w2v_model": self.w2v_model}
 
-    def _set_params(self, params: dict):
+    def udf_set_params(self, params: dict):
         self.w2v_model = params["w2v_model"]
 
 
 class Doc2VecModel(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    ----------------------------
-    output type:pandas.dataframe
-    output like:
-    | 0 | 1 | 2 | 3 |
-    |0.2|0.3|0.4|0.5|
-    |0.3|0.2|0.6|0.1|
+    Doc2Vec词向量模型
     """
 
     def __init__(self, cols="all", embedding_size=16, min_count=5, prefix="d2v", **kwargs):
@@ -369,7 +306,7 @@ class Doc2VecModel(RepresentationBase):
         self.native_init_params.update(self.native_fit_params)
         self.native_init_params.update({"vector_size": embedding_size, "min_count": min_count})
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from gensim.models.doc2vec import Doc2Vec, TaggedDocument
         for col in self.cols:
             texts = s[col].values.tolist()
@@ -379,38 +316,29 @@ class Doc2VecModel(RepresentationBase):
             self.d2v_model[col] = d2v_model
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             d2v_model = self.d2v_model[col]
             texts = s[col].values.tolist()
             texts = [line.split(" ") for line in texts]
             vectors = [d2v_model.infer_vector(line) for line in texts]
-            result = pandas.DataFrame(vectors)
+            result = pd.DataFrame(vectors, index=s.index)
             result.columns = ["{}_{}_{}".format(self.prefix, col, name) for name in result.columns]
-            for icol in result.columns:
-                s[icol] = result[icol].values
-        return s
+            dfs.append(result)
 
-    def _get_params(self) -> dict_type:
+        return PandasUtils.concat_duplicate_columns(dfs)
+
+    def udf_get_params(self) -> dict_type:
         return {"d2v_model": self.d2v_model}
 
-    def _set_params(self, params: dict):
+    def udf_set_params(self, params: dict):
         self.d2v_model = params["d2v_model"]
 
 
 class FastTextModel(RepresentationBase):
     """
-    input type:pandas.series
-    input like:(space separation)
-    |input|
-    |i love eat apple|
-    |i love china|
-    ----------------------------
-    output type:pandas.dataframe
-    output like:
-    | 0 | 1 | 2 | 3 |
-    |0.2|0.3|0.4|0.5|
-    |0.3|0.2|0.6|0.1|
+    FastText词向量模型
     """
 
     def __init__(self, cols="all", embedding_size=16, min_count=5, prefix="fasttext", **kwargs):
@@ -419,7 +347,7 @@ class FastTextModel(RepresentationBase):
         self.native_init_params.update(self.native_fit_params)
         self.native_init_params.update({"vector_size": embedding_size, "min_count": min_count})
 
-    def _fit(self, s: dataframe_type) -> dataframe_type:
+    def udf_fit(self, s: dataframe_type, **kwargs):
         from gensim.models import FastText
         for col in self.cols:
             texts = s[col].values.tolist()
@@ -428,7 +356,8 @@ class FastTextModel(RepresentationBase):
             self.fasttext_model[col] = fasttext_model
         return self
 
-    def _transform(self, s: dataframe_type) -> dataframe_type:
+    def udf_transform(self, s: dataframe_type, **kwargs) -> dataframe_type:
+        dfs = [] if self.drop_input_data else [s]
         for col in self.cols:
             fasttext_model = self.fasttext_model[col]
             texts = s[col].values.tolist()
@@ -437,14 +366,14 @@ class FastTextModel(RepresentationBase):
                                            for word in line if word in fasttext_model.wv]), axis=0) + np.zeros(
                 shape=(self.native_init_params.get("vector_size"),))
                        for line in texts]
-            result = pandas.DataFrame(vectors)
+            result = pd.DataFrame(vectors, index=s.index)
             result.columns = ["{}_{}_{}".format(self.prefix, col, name) for name in result.columns]
-            for icol in result.columns:
-                s[icol] = result[icol].values
-        return s
+            dfs.append(result)
 
-    def _get_params(self) -> dict_type:
+        return PandasUtils.concat_duplicate_columns(dfs)
+
+    def udf_get_params(self) -> dict_type:
         return {"fasttext_model": self.fasttext_model}
 
-    def _set_params(self, params: dict_type):
+    def udf_set_params(self, params: dict_type):
         self.fasttext_model = params["fasttext_model"]
